/*
 * 
 * 
 * Copyright (C) 2005 SIPfoundry Inc.
 * Licensed by SIPfoundry under the LGPL license.
 * 
 * Copyright (C) 2005 Pingtel Corp.
 * Licensed to SIPfoundry under a Contributor Agreement.
 * 
 * $
 */
package org.sipfoundry.sipxconfig.search;

import java.io.IOException;
import java.io.Serializable;

import org.apache.lucene.document.Document;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.index.Term;
import org.hibernate.type.Type;

public class FastIndexer implements Indexer {
    private IndexSource m_indexSource;

    private BeanAdaptor m_beanAdaptor;

    public void indexBean(Object bean, Serializable id, Object[] state, String[] fieldNames,
            Type[] types, boolean newInstance) {
        Document document = new Document();
        if (!m_beanAdaptor.documentFromBean(document, bean, id, state, fieldNames, types)) {
            return;
        }
        if (!newInstance) {
            internalRemoveBean(bean, id);
        }
        addBean(document);
    }

    public void removeBean(Object bean, Serializable id) {
        // only remove beans that are index-able
        if (m_beanAdaptor.indexClass(new Document(), bean.getClass())) {
            internalRemoveBean(bean, id);
        }
    }

    private synchronized void addBean(Document document) {
        IndexWriter writer = null;
        try {
            writer = m_indexSource.getWriter(false);
            writer.addDocument(document);
        } catch (IOException e) {
            throw new RuntimeException(e);
        } finally {
            LuceneUtils.closeQuietly(writer);
        }
    }

    private synchronized void internalRemoveBean(Object bean, Serializable id) {
        IndexReader reader = null;
        try {
            reader = m_indexSource.getReader();
            Term idTerm = m_beanAdaptor.getIdentityTerm(bean, id);
            reader.deleteDocuments(idTerm);
        } catch (IOException e) {
            throw new RuntimeException(e);
        } finally {
            LuceneUtils.closeQuietly(reader);
        }
    }

    public void setBeanAdaptor(BeanAdaptor beanAdaptor) {
        m_beanAdaptor = beanAdaptor;
    }

    public void setIndexSource(IndexSource indexSource) {
        m_indexSource = indexSource;
    }

    public void open() {
        // intentionally empty
    }

    public void close() {
        // intentionally empty
    }
}
